MODEL_METADATA = {
    "classifiers": {
        "adaboost": {
            "label": "AdaBoost",
        },
        "dynamic-nn": {
            "label": "Fully connected neural network (dynamic layer count)",
        },
        "nn-2-layer": {
            "label": "Fully connected neural network (2 hidden layers)",
        },
        "warmstart-nn": {
            "label": "Neural network (warm start, 2 hidden layers)",
        },
        "xgboost": {
            "label": "XGBoost",
        },
    },
    "feature_extractors": {
        "labse": {
            "label": "LaBSE Transformer",
        },
        "mxbai": {
            "label": "mxbai Sentence BERT",
        },
        "sbert": {
            "label": "mpnet Sentence BERT",
        },
        "multilingual-e5-large": {
            "label": "Multilingual E5 Large",
        },
        "gtr-t5-large": {
            "label": "Google GTR",
        },
        "xlm-roberta-large": {
            "label": "XLM RoBERTa Large",
        },
        "doc2vec": {
            "label": "Doc2Vec",
        },
    },
}
